unit CnCodeFormatExpert;

interface

uses
  Classes, ExtCtrls, Forms, Menus, SysUtils, Windows, Dialogs, ToolsApi,
  CnIDEStream, ShellAPI, frmCnCFProcessing, StrUtils, CnWizUtils;

type
  TIDETextExpert = class(TNotifierObject, IOTANotifier, IOTAWizard)
  private
    FCnFormatHost: TMenuItem;
    FCnCodeFormat: TMenuItem;
    FCnCodeOption: TMenuItem;
    procedure OnMCodFormatClick(Sender: TObject);
    procedure OnMCodOptClick(Sender: TObject);
    function GetShortCut: TShortCut;
    procedure SetShortCut(AShortCut: TShortCut);
  protected
  public
    constructor Create; virtual;
    destructor Destroy; override;
    property MenuShortCut: TShortCut read GetShortCut write SetShortCut;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
    procedure Execute;
    procedure ExecuteOption;
    procedure DoFormatFile(AFileName: string);
  end;

var
  IDETextExpert: TIDETextExpert = nil;

implementation

uses
  CnOTAUtils;

{ TIDETextExpert code }

procedure TIDETextExpert.DoFormatFile(AFileName: string);
var
  IDEStream: TIDEStream;
   tmpfile: string;
  str: TStringList;
  ipath: string;
   p: TOTAEditPos;
begin
    p := CnOtaGetTopMostEditView.CursorPos;
    IDEStream := TIDEStream.Create(AFileName);
    ipath := GetInstallPath;
    if ipath[Length(ipath)] <> '\' then
      ipath := ipath + '\';
    tmpfile := ipath + 'tmp';
    if not DirectoryExists(tmpfile) then
      ForceDirectories(tmpfile);
    IDEStream.Lines.SaveToFile(tmpfile + '\Temp');
    ShellExecute(0,'open',PChar(ipath+'CnCodeFormat.exe'),
      PChar('"'+tmpfile + '\Temp"'),PChar(LeftStr(ipath, Length(ipath)-1)), SW_HIDE);
    with TFormCnCFProcessing.Create(Application) do
    begin
      FileName := AFileName;
      ShowModal;
      Free;
    end;
    str := TStringList.Create;
    str.LoadFromFile(tmpfile + '\out_Temp');
    str.Text := DeleteBlankLines(str);
    IDEStream.Lines.Clear;
    IDEStream.WriteText(str.Text);
    IDEStream.Free;
    str.Free;

    // show exception
    if FileExists(tmpfile+'\error.log') then
    begin
      str := TStringList.Create;
      str.LoadFromFile(tmpfile+'\error.log');
      Application.MessageBox(PChar('Syntax error or not supportted, can''t format code.'#13#10+str.Text), 'Error!', MB_OK or MB_ICONINFORMATION);

      {
      Unexpected token, expected "END"
      Row: 15
      Col: 5
      Source: 11
      Desc: Number 11
      }
      str.Text := StringReplace(str.Text,': ','=', [rfReplaceAll, rfIgnoreCase]);
      p.Line := StrToInt(str.ValueFromIndex[1]);
      p.Col := StrToInt(str.ValueFromIndex[2]);

      str.Free;

    end;

    CnOtaGetTopMostEditView.CursorPos := p;
    CnOtaGetTopMostEditView.MoveViewToCursor;
    CnOtaGetTopMostEditView.GetEditWindow.Form.SetFocus;
    // CnOtaGetCurrentSourceEditor.
end;

function TIDETextExpert.GetName: string;
begin
  Result := 'DelForExpert'
end;

function TIDETextExpert.GetState: TWizardState;
begin
  Result := [];
end;

function TIDETextExpert.GetIDString: string;
begin
  Result := 'e_van_nes.DelForExpert';
end;

constructor TIDETextExpert.Create;
var
  FMainMenu: TMainMenu;
  FCnPackMenu: TMenuItem;
  I: integer;
  Ch: Char;
begin
  inherited Create;
  FCnPackMenu := nil;
  // find CnPack's menu
  FMainMenu := (BorlandIDEServices as INTAServices).MainMenu;
  // find Tools
  for i := 0 to FMainMenu.Items.Count - 1 do begin
    if AnsiSameCaption(FMainMenu.Items[i].Name, 'CnPackMenu') then begin
      FCnPackMenu := FMainMenu.Items[i];
      Break;
    end;
  end;

  if Assigned(FCnPackMenu) then
  begin
    FCnFormatHost := TMenuItem.Create(FCnPackMenu);
    FCnFormatHost.Caption := 'Code Formatter';
    // FNewMenuItem.OnClick := OnMenuItemClick;
    FCnPackMenu.Insert(0, FCnFormatHost);
    // sub menu
    FCnCodeFormat := TMenuItem.Create(FCnFormatHost);
    FCnCodeFormat.Caption := 'Format Current Unit';
    FCnCodeFormat.OnClick := OnMCodFormatClick;
    Ch := 'W';
    FCnCodeFormat.ShortCut := ShortCut(Word(Ch), [ssCtrl]);
    FCnFormatHost.Add(FCnCodeFormat);

    FCnCodeOption := TMenuItem.Create(FCnFormatHost);
    FCnCodeOption.Caption := 'Options...';
    FCnCodeOption.OnClick := OnMCodOptClick;
    FCnFormatHost.Add(FCnCodeOption);

  end;
end;

destructor TIDETextExpert.Destroy;
begin
  inherited Destroy;
end;


procedure TIDETextExpert.Execute;
begin
  Self.DoFormatFile('');
end;

procedure TIDETextExpert.ExecuteOption;
var
  cnOpt: string;
begin
  // todo: CAll CnCodeFormatOptions.exe
  cnOpt := GetInstallPath + '\CnCodeFormatOptions.exe';
  ShellExecute(0,'open',
      PChar(cnOpt), nil,
      PChar(GetInstallPath), SW_SHOW);
end;

procedure TIDETextExpert.OnMCodFormatClick(Sender: TObject);
begin
  Execute;
end;

procedure TIDETextExpert.OnMCodOptClick(Sender: TObject);
begin
  ExecuteOption;
end;

function TIDETextExpert.GetShortCut: TShortCut;
begin
  Result := FCnCodeFormat.ShortCut;
end;

procedure TIDETextExpert.SetShortCut(AShortCut: TShortCut);
begin
  FCnCodeFormat.ShortCut := AShortCut;
end;

end.

